# =============================================================
#
# Demo code for training Wasserstein RBMs based on the paper
# 
#  Wasserstein Training of Restricted Boltzmann Machines
#  G. Montavon, KR Mueller, M. Cuturi, NIPS 2016
#
#  Example of execution:
#  > python train.py
#
# =============================================================

import numpy

# path where the MNIST data is located
path = '../data/mnist'

# load MNIST training set
Xf = numpy.fromfile(open('%s/train-images-idx3-ubyte'%path,'r'),dtype='ubyte',count=16+784*60000)[16:].reshape([60000,784]).astype('float32')/255.0
Tf = numpy.fromfile(open('%s/train-labels-idx1-ubyte'%path,'r'),dtype='ubyte',count=8+60000)[8:]

# resize to 14x14
Xf = Xf.reshape([60000,28,28])
Xf = 0.25*(Xf[:,::2,::2]+Xf[:,1::2,::2]+Xf[:,::2,1::2]+Xf[:,1::2,1::2])
Xf = Xf.reshape([60000,196])

# retain only digits "0"
X = Xf[Tf==0]

# binarize at mean threshold
X = (X > X.mean())*1.0

# shuffle the data
X = X[numpy.random.mtrand.RandomState(12345).permutation(len(X))]

# partition in three groups (train/validation/test)
l = len(X)
Xtrain,Xvalid,Xtest = X[:l/3]*1,X[l/3:2*l/3]*1,X[2*l/3:]*1

