# =============================================================
#
# Demo code for training Wasserstein RBMs based on the paper
# 
#  Wasserstein Training of Restricted Boltzmann Machines
#  G. Montavon, KR Mueller, M. Cuturi, NIPS 2016
#
#  Example of execution:
#  > python train.py
#
# =============================================================

import numpy,numpy.linalg,scipy,scipy.io,data
import matplotlib
matplotlib.use('Agg')
from matplotlib import pyplot as plt

# Load data and RBM persistent chains
Xtest = data.Xtest
Xrbm  = scipy.io.loadmat('rbm.mat')['X']

# Compute PCA
m = Xtest.mean(axis=0)
V = numpy.linalg.svd(Xtest-m,full_matrices=0)[2].T

# Visualize test data in PCA space
plt.figure(figsize=(4,4))
PCAtest = numpy.dot(Xtest-m,V)
PCArbm  = numpy.dot(Xrbm-m,V)
plt.plot(PCAtest[:,0],PCAtest[:,1],'.',ms=15,color='gray',alpha=0.1)
plt.plot(PCArbm[:,0],PCArbm[:,1],'.',ms=3,color='red')
plt.axis([-4,4,-4,4])
plt.savefig('pca.png')

# Visualize test data as images
plt.figure(figsize=(4,4))
plt.axis('off')
plt.imshow(Xrbm[:100].reshape([10,10,14,14]).transpose([0,2,1,3]).reshape(10*14,10*14),cmap='gray')
plt.savefig('x-rbm.png')

# Visualize RBM persistent chains as images
plt.figure(figsize=(4,4))
plt.axis('off')
plt.imshow(Xtest[:100].reshape([10,10,14,14]).transpose([0,2,1,3]).reshape(10*14,10*14),cmap='gray')
plt.savefig('x-test.png')

